#ifndef MZSCRIPT_VARIABLES_HPP_INCLUDED
#define MZSCRIPT_VARIABLES_HPP_INCLUDED

#if _MSC_VER > 1020
#  pragma once
#endif // _MSC_VER

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  variables.hpp
//  This is a part of the mzscript source code.
//
//  Copyright (C) 2000-2003 the mzscript developers.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "common.hpp"
#include "utility.hpp"
#include "varfuncs.hpp"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// class Variable
//
class Variable
{
public:
    Variable(const std::string& sValue, const std::string& sFile) :
      m_sValue(sValue), m_pFunc(NULL), m_sFile(sFile), m_cSeparator(':') {}
      
    Variable(VarFunc pFunc) :
      m_pFunc(pFunc), m_cSeparator(':')
        { ASSERT(m_pFunc); }
      
    ~Variable() {}
     
    std::string Get(const std::string& sIndex) const;
    void Set(const std::string& sValue, const std::string& sIndex);
    bool Remove(const std::string& sIndex);

    void SetSeparator(char cSeparator) { m_cSeparator = cSeparator; }
    const std::string& GetFile() const
    {
        return m_sFile;
    }
      
private:
    int _GetElementPos(int nIndex, size_t& stStart, size_t& stEnd) const;

    char m_cSeparator;
    std::string m_sFile;
    mutable std::string m_sValue;
    VarFunc m_pFunc;
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// class Variables
//
// Currently only GetValue is aware of m_pParent
//
class Variables
{
public:
    typedef boost::shared_ptr<Variables> Ptr;
    typedef boost::shared_ptr<Variable> VarPtr;
    
    explicit Variables(Ptr pParent) : m_pParent(pParent) {}
    ~Variables(){}
    
    bool Expand(std::string& sString) const;

    bool Get(std::string sVariable, std::string* psValue) const;
    std::string GetFile(const std::string& sVariable) const;
    VarPtr Set(std::string sVariable, const std::string& sValue,
             const std::string& sFile = std::string());
    bool Set(const std::string& sVariable, VarFunc pFunc);

    bool Remove(std::string sVariable);
    
    void Clear() { m_VarMap.clear(); }
    size_t GetCount() const { return m_VarMap.size(); }
    
private:
    typedef std::map<std::string, VarPtr, stringicmp> VarMap;
    
    bool _GetValueWorker(const std::string& sVariable, std::string* psValue,
        const std::string& sIndex) const;
    bool _ExpandVarsWorker(std::string& sString) const;
    std::string _InterpretVar(std::string& sVariable) const;

    Ptr m_pParent;
    VarMap m_VarMap;
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// "exports"
//
extern Variables::Ptr g_Variables;


#endif // !defined(MZSCRIPT_VARIABLES_HPP_INCLUDED)
